<?php
/**
 * Search criteria for e.g. searching in databases. Do not use this class if
 * you are looking for a ship or company, or what ever. Instead use this class
 * for looking in storages like the database.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class UpdateCriteria extends BaseFrameworkSystem implements LocalUpdateCriteria {
	/**
	 * Criteria to handle
	 */
	private $updateCriteria = array();

	/**
	 * Limitation for the update
	 */
	private $limit = 0;

	/**
	 * Skip these entries before using them
	 */
	private $skip = 0;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("Update criteria class");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Create an instance of this class
	 *
	 * @return	$criteriaInstance	An instance of this criteria
	 */
	public final static function createUpdateCriteria () {
		// Get a new instance
		$criteriaInstance = new UpdateCriteria();

		// Return this instance
		return $criteriaInstance;
	}

	/**
	 * Add criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @return	void
	 */
	public function addCriteria ($criteriaKey, $criteriaValue) {
		$this->updateCriteria[$criteriaKey] = $criteriaValue;
	}

	/**
	 * Add configured criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$configEntry	Configuration entry
	 * @return	void
	 */
	public function addConfiguredCriteria ($criteriaKey, $configEntry) {
		// Add the configuration entry as a criteria
		$value = $this->getConfigInstance()->readConfig($configEntry);
		$this->addCriteria($criteriaKey, $value);
	}

	/**
	 * Getter for update criteria array
	 *
	 * @return	$updateCriteria		Array holding the update criteria
	 */
	public final function getUpdateCriteria () {
		return $this->updateCriteria;
	}
}

// [EOF]
?>
